@@ -175,7 +175,7 @@ static cl_device_partition_property basic_partition_properties[1] = { 0 };
 
 
 static const char *final_ld_flags[] =
-  {"-nostartfiles", HOST_LD_FLAGS_ARRAY, NULL};
+  {"-lm", "-nostartfiles", HOST_LD_FLAGS_ARRAY, NULL};
 
 void
 pocl_init_cpu_device_infos (cl_device_id dev)
