@@ -211,7 +211,11 @@ Workgroup::runOnModule(Module &M) {
           "Target has an unsupported pointer width.");
 
   for (Module::iterator i = M.begin(), e = M.end(); i != e; ++i) {
-    if (!i->isDeclaration())
+    // Don't internalize functions starting with "__wrap_" for the use of GNU
+    // linker's switch --wrap=symbol, where calls to the "symbol" are replaced
+    // with "__wrap_symbol" at link time.  These functions may not be referenced
+    // until final link and being deleted by LLVM optimizations before it.
+    if (!i->isDeclaration() && !i->getName().startswith("__wrap_"))
       i->setLinkage(Function::InternalLinkage);
   }
 
@@ -262,20 +266,6 @@ Workgroup::runOnModule(Module &M) {
         old_kernel->eraseFromParent();
     }
   }
-
-#ifdef LLVM_OLDER_THAN_9_0
-  Function *barrier = cast<Function>
-    (M.getOrInsertFunction(BARRIER_FUNCTION_NAME,
-                           Type::getVoidTy(M.getContext())));
-#else
-  FunctionCallee fc = M.getOrInsertFunction(BARRIER_FUNCTION_NAME,
-                             Type::getVoidTy(M.getContext()));
-  Function *barrier = cast<Function>(fc.getCallee());
-#endif
-
-  BasicBlock *bb = BasicBlock::Create(M.getContext(), "", barrier);
-  ReturnInst::Create(M.getContext(), 0, bb);
-
   return true;
 }
 
@@ -546,8 +536,14 @@ static void replacePrintfCalls(Value *pb, Value *pbp, Value *pbc, bool isKernel,
     CallInst *CI = it.first;
     CallInst *newCI = it.second;
 
-    CI->replaceAllUsesWith(newCI);
-    ReplaceInstWithInst(CI, newCI);
+    // LLVM may modify the result type of the called function to void.
+    if (CI->getType()->isVoidTy()) {
+      newCI->insertBefore(CI);
+      CI->eraseFromParent();
+    } else {
+      CI->replaceAllUsesWith(newCI);
+      ReplaceInstWithInst(CI, newCI);
+    }
   }
 
   replaceCIMap.clear();
@@ -992,7 +988,15 @@ Workgroup::createDefaultWorkgroupLauncher(llvm::Function *F) {
         Arg =
             new llvm::AllocaInst(ArgElementType, ParamType->getAddressSpace(),
                                  ConstantInt::get(IntegerType::get(*C, 32), 1),
-                                 MAX_EXTENDED_ALIGNMENT, "local_auto", Block);
+#ifndef LLVM_OLDER_THAN_10_0
+                                 llvm::MaybeAlign(
+#endif
+                                     MAX_EXTENDED_ALIGNMENT
+#ifndef LLVM_OLDER_THAN_10_0
+                                     )
+#endif
+                                     ,
+                                 "local_auto", Block);
       } else {
         // Dynamic (runtime-set) size local argument.
 
@@ -1006,7 +1010,15 @@ Workgroup::createDefaultWorkgroupLauncher(llvm::Function *F) {
         Value *ElementCount = Builder.CreateUDiv(
             LocalArgByteSize, ConstantInt::get(SizeIntType, ElementSize));
         Arg = new llvm::AllocaInst(ArgElementType, ParamType->getAddressSpace(),
-                                   ElementCount, MAX_EXTENDED_ALIGNMENT,
+                                   ElementCount,
+#ifndef LLVM_OLDER_THAN_10_0
+                                   llvm::MaybeAlign(
+#endif
+                                       MAX_EXTENDED_ALIGNMENT
+#ifndef LLVM_OLDER_THAN_10_0
+                                       )
+#endif
+                                       ,
                                    "local_arg", Block);
       }
     } else {
@@ -1199,7 +1211,15 @@ Workgroup::createArgBufferWorkgroupLauncher(Function *Func,
         // Known static local size (converted automatic local).
         LocalArgAlloca = wrap(new llvm::AllocaInst(
             unwrap(ArgElementType), LLVMGetPointerAddressSpace(ParamType),
-            unwrap(LLVMConstInt(Int32Type, 1, 0)), MAX_EXTENDED_ALIGNMENT,
+            unwrap(LLVMConstInt(Int32Type, 1, 0)),
+#ifndef LLVM_OLDER_THAN_10_0
+            llvm::MaybeAlign(
+#endif
+                MAX_EXTENDED_ALIGNMENT
+#ifndef LLVM_OLDER_THAN_10_0
+                )
+#endif
+                ,
             "local_auto", unwrap(Block)));
       } else {
 
@@ -1227,7 +1247,15 @@ Workgroup::createArgBufferWorkgroupLauncher(Function *Func,
         LocalArgAlloca = wrap(new llvm::AllocaInst(
             unwrap(LLVMGetElementType(ParamType)),
             LLVMGetPointerAddressSpace(ParamType), unwrap(ElementCount),
-            MAX_EXTENDED_ALIGNMENT, "local_arg", unwrap(Block)));
+#ifndef LLVM_OLDER_THAN_10_0
+            llvm::MaybeAlign(
+#endif
+                MAX_EXTENDED_ALIGNMENT
+#ifndef LLVM_OLDER_THAN_10_0
+                )
+#endif
+                ,
+            "local_arg", unwrap(Block)));
       }
       Args[i] = LocalArgAlloca;
     } else {
